CharacterInfo = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
CharacterInfo.version = "0.1." .. string.sub("$Revision: 19718 $", 12, -3);
CharacterInfo.date = string.sub("$Date: 2006-12-09 10:39:23 -0500 (Sat, 09 Dec 2006) $", 8, 17);
CharacterInfo:RegisterDB("CharacterInfoDB");
CharacterInfo:RegisterDefaults('profile', {
	ViewBags = true,
	ViewBank = true,
});

local table_setn
local string_gmatch
local WOW2
do
	local version = GetBuildInfo()
	if string.find(version, "^2%.") then
		-- 2.0.0
		WOW2 = true;
	else
		WOW2 = false;
	end
	if _VERSION == "Lua 5.1" then
		table_setn = function() end
		string_gmatch = string.gmatch
	else
		table_setn = table.setn
		string_gmatch = string.gfind
	end
end

CharacterInfo.vars = {}
CharacterInfo.vars.slotNames = {}
	
local function getIdFromLink(link)
  if not link then return end
  local _,_,id = strfind(link,"item:(%d+):")
  if id then return tonumber(id) end
 	local _,_,id = strfind(link,"enchant:(%d+)")
 	if id then return -tonumber(id) end
end

local function getItemStringFromLink(link)
  if not link then return end
  local _,_,itemString = strfind(link,"item:(%d+:%d+:%d+)")
	return itemString;
end

function CharacterInfo:OnInitialize()
	self:RegisterChatCommand({"/CharacterInfo", "/ci"}, {
		desc = 'View inventory, bags and bank for all your characters',
		type = 'group',
		args = {
			menu = {
				name = 'Menu',
				desc = 'Show CharacterInfo window',
				type = 'execute',
				func = 'Character_Toggle',
			},
			bags = {
				name = 'ViewBags',
				desc = 'View characters bag content',
				type = 'toggle',
				get = function() return self.db.profile.ViewBags end,
				set = function(val) self.db.profile.ViewBags = val end,
			},
			bank = {
				name = 'ViewBank',
				desc = 'View characters bank content (including bank bags)',
				type = 'toggle',
				get = function() return self.db.profile.ViewBank end,
				set = function(val) self.db.profile.ViewBank = val end,
			},
		},
	});
	self.vars.playerName = UnitName("player");
end

function CharacterInfo:OnEnable()
	self.vars.selectedCharacter = self.vars.playerName;
	UIDropDownMenu_SetText(self.vars.selectedCharacter, CharacterInfoCharacterSelection);
end

function CharacterInfo:OnDisable()
end

function CharacterInfo:Character_Show()
	ShowUIPanel(CharacterInfoFrame);
end

function CharacterInfo:Character_Hide()
	HideUIPanel(CharacterInfoFrame);
end

function CharacterInfo:Character_Toggle()
	if CharacterInfoFrame:IsVisible() then
		self:Character_Hide()
	else
		self:Character_Show()
	end
end

function CharacterInfo:Character_OnShow()
	self:Character_Update()
end

function CharacterInfo:Character_OnHide()
	CharacterInfoBagsFrame:Hide();
	CharacterInfoBankFrame:Hide();
end

function CharacterInfo:Bags_OnClick()
	self.db.profile.ViewBags = not self.db.profile.ViewBags;
	self:Character_Update();
end

function CharacterInfo:Bank_OnClick()
	self.db.profile.ViewBank = not self.db.profile.ViewBank;
	self:Character_Update();
end

function CharacterInfo:ItemSlotButton_OnLoad()
--	local text = "ItemSlotButton_OnLoad"..this:GetName()..strsub(this:GetName(),14);
--	DEFAULT_CHAT_FRAME:AddMessage(text);
	local slotName = this:GetName();
	local id, textureName, checkRelic = GetInventorySlotInfo(strsub(slotName,14));
	self.vars.slotNames[id]=slotName;
	this:SetID(id);
	local texture = getglobal(slotName.."IconTexture");
	texture:SetTexture(textureName);
	this.backgroundTextureName = textureName;
	this.checkRelic = checkRelic;
end

function CharacterInfo:ItemSlotButton_OnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	if this.hasItem then
		if this.known then
			GameTooltip:SetHyperlink(this.tooltip);	
		else
			GameTooltip:SetText("Item not in local cache");
			GameTooltip:AddLine("Click to try to update local cache.");
			GameTooltip:AddLine("Warning! You can be disconnected.");
			GameTooltip:Show();
		end
	else
		local text = TEXT(getglobal(strupper(strsub(this:GetName(), 14))));
		if ( this.checkRelic and UnitHasRelicSlot("player") ) then
			text = TEXT(getglobal("RELICSLOT"));
		end
		GameTooltip:SetText(text);
	end	
	CursorUpdate();
end

function CharacterInfo:ContainerItemButton_OnEnter()
	if this.hasItem then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		if this.known then
			GameTooltip:SetHyperlink(this.tooltip);	
		else
			GameTooltip:SetText("Item not in local cache");
			GameTooltip:AddLine("Click to try to update local cache.");
			GameTooltip:AddLine("Warning! You can be disconnected.");
			GameTooltip:Show();
		end
	end	
	CursorUpdate();
end

function CharacterInfo:ItemButton_OnClick(button)
	if this.hasItem then
		if ( button == "LeftButton" ) then
			if ( IsControlKeyDown() ) then
				DressUpItemLink(this.tooltip);
			elseif IsShiftKeyDown() then 
				if ChatFrameEditBox:IsShown() then
					if this.link then
						ChatFrameEditBox:Insert(this.link);
					end
				end
			elseif not this.link then
				GameTooltip:SetOwner(this, "ANCHOR_TOPLEFT");
				GameTooltip:SetHyperlink(this.tooltip);
				self:ScheduleEvent(self.Character_Update, 1, self);
			end
		end
	end
end
---------------------
--Init a Dropdown
function CharacterInfo:DropDown_Initialize()
	--self:Print("DropDown_Initialize",this:GetName());
	local info = {};
	if (this:GetName() == "CharacterInfoCharacterSelectionButton") then
		for name in pairs(CharacterInfoStorage.db.realm) do
			info = {};
			info.text = name;
			info.func = function() self:CharacterDropDown_OnClick() end;
			UIDropDownMenu_AddButton(info);
		end
	end
end

---------------------
-- Dropdown Onclick
function CharacterInfo:CharacterDropDown_OnClick()
	--self:Print("CharacterDropDown_OnClick",this:GetID());
	local ddl = getglobal("CharacterInfoCharacterSelection");
	UIDropDownMenu_SetSelectedID(ddl, this:GetID());
	self.vars.selectedCharacter = this:GetText();
	self:Character_Update();
end

-- Draw windows for a character
function CharacterInfo:Character_Update()
	CharacterInfoNameText:SetText(self.vars.selectedCharacter);
	CharacterInfoLevelText:SetText(	string.format("Level %s %s %s", 
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].level,
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].race,
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].class ));
	for i=1,19 do
		local itemSlot = getglobal(self.vars.slotNames[i]);
		local name,itemString,link,texture,known,num = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[i]);
		if name then
			SetItemButtonTexture(itemSlot,texture);
			itemSlot.hasItem=true;
			itemSlot.tooltip = itemString;
			itemSlot.link = link;
			itemSlot.known = known;
		else
			itemSlot.hasItem=false;
			itemSlot.link = nil;
			itemSlot.known = nil;
			SetItemButtonTexture(itemSlot,itemSlot.backgroundTextureName);
		end
	end	
	if self.db.profile.ViewBags then
		self:DrawContainer(0);
	else
		CharacterInfoBagsFrame:Hide();
	end
	if self.db.profile.ViewBank then
		self:DrawContainer(1);
	else
		CharacterInfoBankFrame:Hide();
	end
end

function CharacterInfo:GetItemInfo(inf)
	if not inf then return end
	local known = true;
	local _,_,id,num = string.find(inf,"([^|]+)[|]?(%d*)");
	if num and num~="" then num = tonumber(num) else num = nil end
	local itemString = "item:"..id..":0:0:0:0:0";
	local name,itemLink,quality,texture
	name,itemLink,quality,_,_,_,_,_,_,texture = GetItemInfo("item:"..id..":0:0:0:0:0");
	if not name then
		name = "Unknown Item";
		itemLink = nil;
		texture = "Interface\\Icons\\INV_Misc_QuestionMark.blp"
		known = false;
	end
	return name,itemString,itemLink,texture,known,num;
end

function CharacterInfo:GetItemButton(frame,index)
	local itemButton = getglobal(frame:GetName().."Item"..index);
	if not itemButton then
		itemButton = CreateFrame("Button", frame:GetName().."Item"..index, frame, "CharacterInfoContainerButtonTemplate");
	end
	itemButton:ClearAllPoints();
	return itemButton
end

function CharacterInfo:SetItemButtomItem(itemButton, item)
	if item then
		local name,itemString,link,texture,known,num = self:GetItemInfo(item);
		SetItemButtonTexture(itemButton, texture);
		SetItemButtonCount(itemButton, num);
		itemButton.hasItem = true;
		itemButton.tooltip = itemString;
		itemButton.link = link;
		itemButton.known = known;
	else
		SetItemButtonTexture(itemButton, "");
		SetItemButtonCount(itemButton, nil);
		itemButton.hasItem = false;
		itemButton.tooltip = nil;
		itemButton.link = nil;
		itemButton.known = nil;
	end
	itemButton:Show();
end

function CharacterInfo:DrawContainer(id)
	local frame;
	local frameName;
	local container;
	local size;
	if ( id == 0 ) then  -- Bags
		frame = getglobal("CharacterInfoBagsFrame");
		frameName = frame:GetName();
		getglobal(frameName.."Name"):SetText("Bags");
		container = CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bags;
		size = 16;
		for j=1, 4 do
			local _,_,_,_,_,num = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[ContainerIDToInventoryID(j)]);
			if num then size = size + num end
		end
	elseif ( id == 1 ) then  -- Bank
		frame = getglobal("CharacterInfoBankFrame");
		frameName = frame:GetName();
		getglobal(frameName.."Name"):SetText("Bank");
		container = CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank;
		size = 28;
		for j=5, 11 do
			local _,_,_,_,_,num = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank[ContainerIDToInventoryID(j)]);
			if num then size = size + num end
		end
	else
		return;
	end
	local items = {};
	for key in pairs(container) do
		if key < 60 or key > 100 then
			table.insert(items,key);
		end
	end
	table.sort(items,function(a,b) return tonumber(a)<tonumber(b) end);
	local rows = ceil(math.sqrt(size));
	local columns = ceil(size / rows);
	if ( columns < 4 ) then 
		columns = 4;
		rows = ceil(size / columns);
	end
	if ( id == 1 and columns < 7 ) then 
		columns = 7;
		rows = ceil(size / columns);
	 end
	--self:Print("DrawContainer",size,columns,rows);
	frame:SetHeight(rows * 42 + 26 + 48);
  frame:SetWidth(columns * 42 + 12);
	for j=1,size do
		local itemButton = self:GetItemButton(frame,j);
		-- Set first button
		if( j == 1 ) then
			itemButton:SetPoint("BOTTOMRIGHT", frameName, "BOTTOMRIGHT", -8, 8+48);
		else
			if ( mod((j-1), columns) == 0 ) then
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j - columns), "TOPRIGHT", 0, 5);
			else
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j - 1), "BOTTOMLEFT", -5, 0);
			end
		end
		if ( items[j] ) then
			self:SetItemButtomItem(itemButton, container[items[j]]);
		else
			self:SetItemButtomItem(itemButton);
		end
	end
	if ( id == 0 ) then
		for j=1,4 do
			local itemButton = self:GetItemButton(frame,j+size);
			if ( j == 1 ) then
				itemButton:SetPoint("BOTTOMRIGHT", frameName, "BOTTOMRIGHT", -8, 8);
			else
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j + size - 1), "BOTTOMLEFT", -5, 0);
			end
			self:SetItemButtomItem(itemButton, CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[ContainerIDToInventoryID(j)]);
		end
	elseif ( id == 1 ) then
		for j=1,7 do
			local itemButton = self:GetItemButton(frame,j+size);
			if ( j == 1 ) then
				itemButton:SetPoint("BOTTOMLEFT", frameName, "BOTTOMLEFT", 8, 8);
			else
				itemButton:SetPoint("BOTTOMLEFT", frameName.."Item"..(j + size - 1), "BOTTOMRIGHT", 5, 0);
			end
			self:SetItemButtomItem(itemButton, CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank[ContainerIDToInventoryID(4+j)]);
		end
	end
	j = size + (((id == 0) and 5) or ((id == 1) and 8));
	while true do
		local itemButton = getglobal(frameName.."Item"..j);
		if itemButton then
			itemButton:Hide();
		else
			break;
		end
		j = j + 1;
	end
	if ( id == 0 ) then
		frame:SetPoint("TOPLEFT", "CharacterInfoFrame", "TOPRIGHT", 4, 0 );
	elseif ( id == 1 ) then
		if CharacterInfoBagsFrame:IsVisible() then
			frame:SetPoint("TOPLEFT", "CharacterInfoBagsFrame", "TOPRIGHT", 4, 0 );
		else
			frame:SetPoint("TOPLEFT", "CharacterInfoFrame", "TOPRIGHT", 4, 0 );
		end
	end
	frame:Show();
end
